@@ -1,18 +1,29 @@
 #' @title Calculate new pH and ion balance after chemical addition
 #'
-#' @description \code{chemdose_ph} calculates the new pH, alkalinity, and ion balance of a water based on different chemical
+#' @description Calculates the new pH, alkalinity, and ion balance of a water based on different chemical
 #' additions.
+#' For a single water use `chemdose_ph`; for a dataframe use `chemdose_ph_chain`.
+#' Use [pluck_water] to get values from the output water as new dataframe columns.
+#' For most arguments in the `_chain` helper
+#' "use_col" default looks for a column of the same name in the dataframe. The argument can be specified directly in the
+#' function instead or an unquoted column name can be provided.
 #'
 #' @details The function takes an object of class "water" created by \code{\link{define_water}} and user-specified
 #' chemical additions and returns a new object of class "water" with updated water quality.
 #' Units of all chemical additions are in mg/L as chemical (not as product).
 #'
-#' \code{chemdose_ph} works by evaluating all the user-specified chemical additions and solving for what the new pH
-#' must be using \code{uniroot} to satisfy the principle of electroneutrality in pure water while correcting for the existing alkalinity
+#' `chemdose_ph` works by evaluating all the user-specified chemical additions and solving for what the new pH
+#' must be using [uniroot] to satisfy the principle of electroneutrality in pure water while correcting for the existing alkalinity
 #' of the water that the chemical is added to. Multiple chemicals can be added simultaneously or each addition can be
 #' modeled independently through sequential doses.
 #'
-#' @param water Source water object of class "water" created by \code{\link{define_water}}
+#' For large datasets, using `fn_once` or `fn_chain` may take many minutes to run. These types of functions use the furrr package
+#'  for the option to use parallel processing and speed things up. To initialize parallel processing, use
+#'  `plan(multisession)` or `plan(multicore)` (depending on your operating system) prior to your piped code with the
+#'  `fn_once` or `fn_chain` functions. Note, parallel processing is best used when your code block takes more than a minute to run,
+#'  shorter run times will not benefit from parallel processing.
+#'
+#' @param water Source water object of class "water" created by [define_water]
 #' @param hcl Amount of hydrochloric acid added in mg/L: HCl -> H + Cl
 #' @param h2so4 Amount of sulfuric acid added in mg/L: H2SO4 -> 2H + SO4
 #' @param h3po4 Amount of phosphoric acid added in mg/L: H3PO4 -> 3H + PO4
@@ -61,7 +72,7 @@
 #'
 #' @export
 #'
-#' @returns A water class object with updated pH, alkalinity, and ions post-chemical addition.
+#' @returns `chemdose_ph` returns a water class object with updated pH, alkalinity, and ions post-chemical addition.
 #'
 chemdose_ph <- function(water, hcl = 0, h2so4 = 0, h3po4 = 0, co2 = 0,
                         naoh = 0, caoh2 = 0, mgoh2 = 0,
@@ -69,6 +80,17 @@ chemdose_ph <- function(water, hcl = 0, h2so4 = 0, h3po4 = 0, co2 = 0,
                         cl2 = 0, naocl = 0, nh4oh = 0, nh42so4 = 0,
                         alum = 0, ferricchloride = 0, ferricsulfate = 0, ach = 0,
                         softening_correction = FALSE) {
+  if ((cacl2 > 0 | cl2 > 0 | naocl > 0) & (nh4oh > 0 | nh42so4 > 0)) {
+    warning("Both chlorine- and ammonia-based chemicals were dosed and may form chloramines.\nUse chemdose_chloramine for breakpoint caclulations.")
+  }
+  if ((cacl2 > 0 | cl2 > 0 | naocl > 0) & water@tot_nh3 > 0) {
+    warning("A chlorine-based chemical was dosed into a water containing ammonia, which may form chloramines.\nUse chemdose_chloramine for breakpoint caclulations.")
+  }
+
+  if ((nh4oh > 0 | nh42so4 > 0) & (water@free_chlorine > 0 | water@combined_chlorine > 0)) {
+    warning("An ammonia-based chemical was dosed into a water containing chlorine, which may form chloramines.\nUse chemdose_chloramine for breakpoint caclulations.")
+  }
+
   validate_water(water, c("ph", "alk"))
 
   #### CONVERT INDIVIDUAL CHEMICAL ADDITIONS TO MOLAR ####
@@ -179,8 +201,9 @@ chemdose_ph <- function(water, hcl = 0, h2so4 = 0, h3po4 = 0, co2 = 0,
     ph <- ph_corrected
   }
 
-  h <- 10^-ph
-  oh <- dosed_water@kw / h
+  # Convert from pH to concentration (not activity)
+  h <- (10^-ph) / calculate_activity(1, water@is, water@temp)
+  oh <- dosed_water@kw / (h * calculate_activity(1, water@is, water@temp)^2)
 
   # Correct eq constants
   ks <- correct_k(dosed_water)
@@ -218,181 +241,37 @@ chemdose_ph <- function(water, hcl = 0, h2so4 = 0, h3po4 = 0, co2 = 0,
   return(dosed_water)
 }
 
-#' Apply `chemdose_ph` function and output a dataframe
-#'
-#' This function allows \code{\link{chemdose_ph}} to be added to a piped data frame.
-#' Its output is a data frame with updated ions and pH.
-#'
-#' The data input comes from a `water` class column, as initialized in \code{\link{define_water}} or \code{\link{balance_ions}}.
-#'
-#' If the input data frame has a column(s) name matching a valid chemical(s), the function will dose that chemical(s) in addition to the
-#' ones specified in the function's arguments.
-#' The column names must match the chemical names as displayed in \code{\link{chemdose_ph}}.
-#' To see which chemicals can be passed into the function, see \code{\link{chemdose_ph}}.
-#'
-#' tidywater functions cannot be added after this function because they require a `water` class input.
-#'
-#'  For large datasets, using `fn_once` or `fn_chain` may take many minutes to run. These types of functions use the furrr package
-#'  for the option to use parallel processing and speed things up. To initialize parallel processing, use
-#'  `plan(multisession)` or `plan(multicore)` (depending on your operating system) prior to your piped code with the
-#'  `fn_once` or `fn_chain` functions. Note, parallel processing is best used when your code block takes more than a minute to run,
-#'  shorter run times will not benefit from parallel processing.
-#'
+#' @rdname chemdose_ph
 #' @param df a data frame containing a water class column, which has already been computed using
-#' \code{\link{define_water_chain}}. The df may include columns named for the chemical(s) being dosed.
-#' @param input_water name of the column of water class data to be used as the input for this function. Default is "defined_water".
-#' @param hcl Hydrochloric acid: HCl -> H + Cl
-#' @param h2so4 Sulfuric acid: H2SO4 -> 2H + SO4
-#' @param h3po4 Phosphoric acid: H3PO4 -> 3H + PO4
-#' @param co2 Carbon Dioxide CO2 (gas) + H2O -> H2CO3*
-#' @param naoh Caustic: NaOH -> Na + OH
-#' @param na2co3 Soda ash: Na2CO3 -> 2Na + CO3
-#' @param nahco3 Sodium bicarbonate: NaHCO3 -> Na + H + CO3
-#' @param caoh2 Lime: Ca(OH)2 -> Ca + 2OH
-#' @param mgoh2  Magneisum hydroxide: Mg(OH)2 -> Mg + 2OH
-#' @param cl2 Chlorine gas: Cl2(g) + H2O -> HOCl + H + Cl
-#' @param naocl Sodium hypochlorite: NaOCl -> Na + OCl
-#' @param nh4oh Amount of ammonium hydroxide added in mg/L as N: NH4OH -> NH4 + OH
-#' @param nh42so4 Amount of ammonium sulfate added in mg/L as N: (NH4)2SO4 -> 2NH4 + SO4
-#' @param alum Hydrated aluminum sulfate Al2(SO4)3*14H2O + 6HCO3 -> 2Al(OH)3(am) +3SO4 + 14H2O + 6CO2
-#' @param ferricchloride Ferric Chloride FeCl3 + 3HCO3 -> Fe(OH)3(am) + 3Cl + 3CO2
-#' @param ferricsulfate Amount of ferric sulfate added in mg/L: Fe2(SO4)3*8.8H2O + 6HCO3 -> 2Fe(OH)3(am) + 3SO4 + 8.8H2O + 6CO2
-#' @param ach Amount of aluminum chlorohydrate added in mg/L: Al2(OH)5Cl*2H2O + HCO3 -> 2Al(OH)3(am) + Cl + 2H2O + CO2
-#' @param caco3 Amount of calcium carbonate added (or removed) in mg/L: CaCO3 -> Ca + CO3
-#'
-#' @seealso \code{\link{chemdose_ph}}
-#'
-#' @examples
-#'
-#' library(purrr)
-#' library(furrr)
-#' library(tidyr)
-#' library(dplyr)
-#'
-#' example_df <- water_df %>%
-#'   define_water_chain() %>%
-#'   balance_ions_chain() %>%
-#'   chemdose_ph_once(input_water = "balanced_water", naoh = 5)
-#'
-#' example_df <- water_df %>%
-#'   define_water_chain() %>%
-#'   balance_ions_chain() %>%
-#'   mutate(
-#'     hcl = seq(1, 12, 1),
-#'     naoh = 20
-#'   ) %>%
-#'   chemdose_ph_once(input_water = "balanced_water", mgoh2 = 55, co2 = 4)
-#'
-#' \donttest{
-#' # Initialize parallel processing
-#' plan(multisession, workers = 2) # Remove the workers argument to use all available compute
-#' example_df <- water_df %>%
-#'   define_water_chain() %>%
-#'   balance_ions_chain() %>%
-#'   chemdose_ph_once(input_water = "balanced_water", naoh = 5)
-#'
-#' # Optional: explicitly close multisession processing
-#' plan(sequential)
-#' }
-#'
-#' @import dplyr
-#' @importFrom tidyr unnest
-#' @export
-#'
-#' @returns A data frame with updated pH, alkalinity, and ions post-chemical addition.
-#'
-
-chemdose_ph_once <- function(df, input_water = "defined_water",
-                             hcl = 0, h2so4 = 0, h3po4 = 0, co2 = 0, naoh = 0,
-                             na2co3 = 0, nahco3 = 0, caoh2 = 0, mgoh2 = 0,
-                             cl2 = 0, naocl = 0, nh4oh = 0, nh42so4 = 0,
-                             alum = 0, ferricchloride = 0, ferricsulfate = 0, ach = 0, caco3 = 0) {
-  dose_chem <- dosed_chem_water <- NULL # Quiet RCMD check global variable note
-  output <- df %>%
-    chemdose_ph_chain(
-      input_water = input_water, output_water = "dosed_chem_water",
-      hcl, h2so4, h3po4, co2, naoh,
-      na2co3, nahco3, caoh2, mgoh2,
-      cl2, naocl, nh4oh, nh42so4,
-      alum, ferricchloride, ferricsulfate, ach, caco3
-    ) %>%
-    mutate(dose_chem = furrr::future_map(dosed_chem_water, convert_water)) %>%
-    unnest(dose_chem) %>%
-    select(-dosed_chem_water)
-}
-
-#' Apply `chemdose_ph` within a dataframe and output a column of `water` class to be chained to other tidywater functions
-#'
-#' This function allows \code{\link{chemdose_ph}} to be added to a piped data frame.
-#' Its output is a `water` class, and can therefore be used with "downstream" tidywater functions.
-#' Ions and pH will be updated based on input chemical doses.
-#'
-#' The data input comes from a `water` class column, as initialized in \code{\link{define_water}} or \code{\link{balance_ions}}.
-#'
-#' If the input data frame has a column(s) name matching a valid chemical(s), the function will dose that chemical(s) in addition to the
-#' ones specified in the function's arguments.
-#' The column names must match the chemical names as displayed in \code{\link{chemdose_ph}}.
-#' To see which chemicals can be passed into the function, see \code{\link{chemdose_ph}}.
-#'
-#'  For large datasets, using `fn_once` or `fn_chain` may take many minutes to run. These types of functions use the furrr package
-#'  for the option to use parallel processing and speed things up. To initialize parallel processing, use
-#'  `plan(multisession)` or `plan(multicore)` (depending on your operating system) prior to your piped code with the
-#'  `fn_once` or `fn_chain` functions. Note, parallel processing is best used when your code block takes more than a minute to run,
-#'  shorter run times will not benefit from parallel processing.
-#'
-#' @param df a data frame containing a water class column, which has already been computed using
-#' \code{\link{define_water_chain}}. The df may include columns named for the chemical(s) being dosed.
+#' [define_water_chain] The df may include columns named for the chemical(s) being dosed.
 #' @param input_water name of the column of water class data to be used as the input for this function. Default is "defined_water".
 #' @param output_water name of the output column storing updated parameters with the class, water. Default is "dosed_chem_water".
-#' @param hcl Hydrochloric acid: HCl -> H + Cl
-#' @param h2so4 Sulfuric acid: H2SO4 -> 2H + SO4
-#' @param h3po4 Phosphoric acid: H3PO4 -> 3H + PO4
-#' @param co2 Carbon Dioxide CO2 (gas) + H2O -> H2CO3*
-#' @param naoh Caustic: NaOH -> Na + OH
-#' @param na2co3 Soda ash: Na2CO3 -> 2Na + CO3
-#' @param nahco3 Sodium bicarbonate: NaHCO3 -> Na + H + CO3
-#' @param caoh2 Lime: Ca(OH)2 -> Ca + 2OH
-#' @param mgoh2  Magneisum hydroxide: Mg(OH)2 -> Mg + 2OH
-#' @param cl2 Chlorine gas: Cl2(g) + H2O -> HOCl + H + Cl
-#' @param naocl Sodium hypochlorite: NaOCl -> Na + OCl
-#' @param nh4oh Amount of ammonium hydroxide added in mg/L as N: NH4OH -> NH4 + OH
-#' @param nh42so4 Amount of ammonium sulfate added in mg/L as N: (NH4)2SO4 -> 2NH4 + SO4
-#' @param alum Hydrated aluminum sulfate Al2(SO4)3*14H2O + 6HCO3 -> 2Al(OH)3(am) +3SO4 + 14H2O + 6CO2
-#' @param ferricchloride Ferric Chloride FeCl3 + 3HCO3 -> Fe(OH)3(am) + 3Cl + 3CO2
-#' @param ferricsulfate Amount of ferric sulfate added in mg/L: Fe2(SO4)3*8.8H2O + 6HCO3 -> 2Fe(OH)3(am) + 3SO4 + 8.8H2O + 6CO2
-#' @param ach Amount of aluminum chlorohydrate added in mg/L: Al2(OH)5Cl*2H2O + HCO3 -> 2Al(OH)3(am) + Cl + 2H2O + CO2
-#' @param caco3 Amount of calcium carbonate added (or removed) in mg/L: CaCO3 -> Ca + CO3
-#'
-#' @seealso \code{\link{chemdose_ph}}
 #'
 #' @examples
 #'
-#' library(purrr)
-#' library(furrr)
-#' library(tidyr)
 #' library(dplyr)
 #'
 #' example_df <- water_df %>%
+#'   slice_head(n = 3) %>%
 #'   define_water_chain() %>%
-#'   balance_ions_chain() %>%
-#'   chemdose_ph_chain(input_water = "balanced_water", naoh = 5)
+#'   chemdose_ph_chain(input_water = "defined_water", naoh = 5)
 #'
 #' example_df <- water_df %>%
+#'   slice_head(n = 3) %>%
 #'   define_water_chain() %>%
-#'   balance_ions_chain() %>%
 #'   mutate(
-#'     hcl = seq(1, 12, 1),
-#'     naoh = 20
+#'     hcl = c(2, 4, 6),
+#'     Caustic = 20
 #'   ) %>%
-#'   chemdose_ph_chain(input_water = "balanced_water", mgoh2 = 55, co2 = 4)
+#'   chemdose_ph_chain(mgoh2 = c(20, 55), co2 = 4, naoh = Caustic)
 #'
 #' \donttest{
 #' # Initialize parallel processing
+#' library(furrr)
 #' plan(multisession, workers = 2) # Remove the workers argument to use all available compute
 #' example_df <- water_df %>%
 #'   define_water_chain() %>%
-#'   balance_ions_chain() %>%
-#'   chemdose_ph_chain(input_water = "balanced_water", naoh = 5)
+#'   chemdose_ph_chain(naoh = 5)
 #'
 #' # Optional: explicitly close multisession processing
 #' plan(sequential)
@@ -401,84 +280,84 @@ chemdose_ph_once <- function(df, input_water = "defined_water",
 #' @import dplyr
 #' @export
 #'
-#' @returns A data frame containing a water class column with updated pH, alkalinity, and ions post-chemical addition.
+#' @returns `chemdose_ph_chain` returns a data frame containing a water class column with updated pH, alkalinity, and ions post-chemical addition.
 
 chemdose_ph_chain <- function(df, input_water = "defined_water", output_water = "dosed_chem_water",
-                              hcl = 0, h2so4 = 0, h3po4 = 0, co2 = 0, naoh = 0,
-                              na2co3 = 0, nahco3 = 0, caoh2 = 0, mgoh2 = 0,
-                              cl2 = 0, naocl = 0, nh4oh = 0, nh42so4 = 0,
-                              alum = 0, ferricchloride = 0, ferricsulfate = 0, ach = 0, caco3 = 0) {
-  ID <- NULL # Quiet RCMD check global variable note
-  dosable_chems <- tibble(
-    hcl, h2so4, h3po4, co2, naoh,
-    na2co3, nahco3, caoh2, mgoh2,
-    cl2, naocl, nh4oh, nh42so4,
-    alum, ferricchloride, ferricsulfate, ach, caco3
-  )
+                              hcl = "use_col", h2so4 = "use_col", h3po4 = "use_col", co2 = "use_col", naoh = "use_col",
+                              na2co3 = "use_col", nahco3 = "use_col", caoh2 = "use_col", mgoh2 = "use_col",
+                              cacl2 = "use_col", cl2 = "use_col", naocl = "use_col",
+                              nh4oh = "use_col", nh42so4 = "use_col",
+                              alum = "use_col", ferricchloride = "use_col", ferricsulfate = "use_col", ach = "use_col",
+                              caco3 = "use_col", softening_correction = "use_col") {
+  validate_water_helpers(df, input_water)
+  # This allows for the function to process unquoted column names without erroring
+  hcl <- tryCatch(hcl, error = function(e) enquo(hcl))
+  h2so4 <- tryCatch(h2so4, error = function(e) enquo(h2so4))
+  h3po4 <- tryCatch(h3po4, error = function(e) enquo(h3po4))
+  co2 <- tryCatch(co2, error = function(e) enquo(co2))
+  naoh <- tryCatch(naoh, error = function(e) enquo(naoh))
 
-  chem_inputs_arg <- dosable_chems %>%
-    select_if(~ any(. > 0))
+  na2co3 <- tryCatch(na2co3, error = function(e) enquo(na2co3))
+  nahco3 <- tryCatch(nahco3, error = function(e) enquo(nahco3))
+  caoh2 <- tryCatch(caoh2, error = function(e) enquo(caoh2))
+  mgoh2 <- tryCatch(mgoh2, error = function(e) enquo(mgoh2))
 
-  chem_inputs_col <- df %>%
-    subset(select = names(df) %in% names(dosable_chems)) %>%
-    # add row number for joining
-    mutate(ID = row_number())
+  cacl2 <- tryCatch(cacl2, error = function(e) enquo(cacl2))
+  cl2 <- tryCatch(cl2, error = function(e) enquo(cl2))
+  naocl <- tryCatch(naocl, error = function(e) enquo(naocl))
 
-  if (length(chem_inputs_col) - 1 == 0 & length(chem_inputs_arg) == 0) {
-    warning("No chemical dose found. Create dose column, enter a dose argument, or check availbility of chemical in the chemdose_ph function.")
+  nh4oh <- tryCatch(nh4oh, error = function(e) enquo(nh4oh))
+  nh42so4 <- tryCatch(nh42so4, error = function(e) enquo(nh42so4))
+
+  alum <- tryCatch(alum, error = function(e) enquo(alum))
+  ferricchloride <- tryCatch(ferricchloride, error = function(e) enquo(ferricchloride))
+  ferricsulfate <- tryCatch(ferricsulfate, error = function(e) enquo(ferricsulfate))
+  ach <- tryCatch(ach, error = function(e) enquo(ach))
+  caco3 <- tryCatch(caco3, error = function(e) enquo(caco3))
+
+  softening_correction <- tryCatch(softening_correction, error = function(e) enquo(softening_correction))
+
+  # This returns a dataframe of the input arguments and the correct column names for the others
+  arguments <- construct_helper(df, all_args = list(
+    "hcl" = hcl, "h2so4" = h2so4, "h3po4" = h3po4, "co2" = co2, "naoh" = naoh,
+    "na2co3" = na2co3, "nahco3" = nahco3, "caoh2" = caoh2, "mgoh2" = mgoh2,
+    "cacl2" = cacl2, "cl2" = cl2, "naocl" = naocl,
+    "nh4oh" = nh4oh, "nh42so4" = nh42so4,
+    "alum" = alum, "ferricchloride" = ferricchloride, "ferricsulfate" = ferricsulfate, "ach" = ach, "caco3" = caco3,
+    "softening_correction" = softening_correction
+  ))
+  final_names <- arguments$final_names
+
+  # Only join inputs if they aren't in existing dataframe
+  if (length(arguments$new_cols) > 0) {
+    df <- df %>%
+      cross_join(as.data.frame(arguments$new_cols))
   }
-
-  if (length(chem_inputs_col) > 0 & length(chem_inputs_arg) > 0) {
-    if (any(names(chem_inputs_arg) %in% names(chem_inputs_col))) {
-      stop("At least one chemical was dosed as both a function argument and a data frame column. Remove your chemical(s) from one of these inputs.")
-    }
-  }
-
-  if (nrow(chem_inputs_arg) == 1) {
-    chem_doses <- chem_inputs_col %>%
-      cross_join(chem_inputs_arg)
-    # Add missing chemical columns
-    chem2 <- dosable_chems %>%
-      subset(select = !names(dosable_chems) %in% names(chem_doses)) %>%
-      cross_join(chem_doses) %>%
-      mutate(ID = row_number())
-  } else if (nrow(chem_inputs_arg) > 1) {
-    chem_doses <- chem_inputs_col %>%
-      cross_join(chem_inputs_arg)
-    chem2 <- dosable_chems %>%
-      subset(select = !names(dosable_chems) %in% names(chem_doses)) %>%
-      unique() %>%
-      cross_join(chem_doses)
-  }
-
   output <- df %>%
-    subset(select = !names(df) %in% names(chem_inputs_col)) %>%
-    mutate(ID = row_number()) %>%
-    left_join(chem2, by = "ID") %>%
-    select(-ID) %>%
     mutate(!!output_water := furrr::future_pmap(
       list(
         water = !!as.name(input_water),
-        hcl = hcl,
-        h2so4 = h2so4,
-        h3po4 = h3po4,
-        co2 = co2,
-        naoh = naoh,
-        na2co3 = na2co3,
-        nahco3 = nahco3,
-        caoh2 = caoh2,
-        mgoh2 = mgoh2,
-        cl2 = cl2,
-        naocl = naocl,
-        nh4oh = nh4oh,
-        nh42so4 = nh4oh,
-        alum = alum,
-        ferricchloride = ferricchloride,
-        ferricsulfate = ferricsulfate,
-        ach = ach,
-        caco3 = caco3
+        hcl = if (final_names$hcl %in% names(.)) !!sym(final_names$hcl) else rep(0, nrow(.)),
+        h2so4 = if (final_names$h2so4 %in% names(.)) !!sym(final_names$h2so4) else rep(0, nrow(.)),
+        h3po4 = if (final_names$h3po4 %in% names(.)) !!sym(final_names$h3po4) else rep(0, nrow(.)),
+        co2 = if (final_names$co2 %in% names(.)) !!sym(final_names$co2) else rep(0, nrow(.)),
+        naoh = if (final_names$naoh %in% names(.)) !!sym(final_names$naoh) else rep(0, nrow(.)),
+        na2co3 = if (final_names$na2co3 %in% names(.)) !!sym(final_names$na2co3) else rep(0, nrow(.)),
+        nahco3 = if (final_names$nahco3 %in% names(.)) !!sym(final_names$nahco3) else rep(0, nrow(.)),
+        caoh2 = if (final_names$caoh2 %in% names(.)) !!sym(final_names$caoh2) else rep(0, nrow(.)),
+        mgoh2 = if (final_names$mgoh2 %in% names(.)) !!sym(final_names$mgoh2) else rep(0, nrow(.)),
+        cacl2 = if (final_names$cacl2 %in% names(.)) !!sym(final_names$cacl2) else rep(0, nrow(.)),
+        cl2 = if (final_names$cl2 %in% names(.)) !!sym(final_names$cl2) else rep(0, nrow(.)),
+        naocl = if (final_names$naocl %in% names(.)) !!sym(final_names$naocl) else rep(0, nrow(.)),
+        nh4oh = if (final_names$nh4oh %in% names(.)) !!sym(final_names$nh4oh) else rep(0, nrow(.)),
+        nh42so4 = if (final_names$nh42so4 %in% names(.)) !!sym(final_names$nh42so4) else rep(0, nrow(.)),
+        alum = if (final_names$alum %in% names(.)) !!sym(final_names$alum) else rep(0, nrow(.)),
+        ferricchloride = if (final_names$ferricchloride %in% names(.)) !!sym(final_names$ferricchloride) else rep(0, nrow(.)),
+        ferricsulfate = if (final_names$ferricsulfate %in% names(.)) !!sym(final_names$ferricsulfate) else rep(0, nrow(.)),
+        ach = if (final_names$ach %in% names(.)) !!sym(final_names$ach) else rep(0, nrow(.)),
+        caco3 = if (final_names$caco3 %in% names(.)) !!sym(final_names$caco3) else rep(0, nrow(.)),
+        softening_correction = if (final_names$softening_correction %in% names(.)) !!sym(final_names$softening_correction) else rep(FALSE, nrow(.))
       ),
       chemdose_ph
-    )) %>%
-    select(!any_of(names(dosable_chems)), any_of(names(chem_doses)))
+    ))
 }
