@@ -4,27 +4,38 @@
 #' @title Calculate bromate formation
 #'
 #' @description Calculates bromate (BrO3-, ug/L) formation based on selected model. Required arguments include an object of class "water"
-#' created by \code{\link{define_water}} ozone dose, reaction time, and desired model.
-#' The function also requires additional water quality parameters defined in \code{define_water}
+#' created by [define_water] ozone dose, reaction time, and desired model.
+#' The function also requires additional water quality parameters defined in [define_water]
 #' including bromide, DOC or UV254 (depending on the model), pH, alkalinity (depending on the model), and
 #' optionally, ammonia (added when defining water using the `tot_nh3` argument.)
+#' For a single water use `ozonate_bromate`; for a dataframe use `ozonate_bromate_chain`.
+#' Use [pluck_water] to get values from the output water as new dataframe columns.
+#' For most arguments in the `_chain` helper
+#' "use_col" default looks for a column of the same name in the dataframe. The argument can be specified directly in the
+#' function instead or an unquoted column name can be provided.
 #'
+#' @details
+#' For large datasets, using `fn_once` or `fn_chain` may take many minutes to run. These types of functions use the furrr package
+#'  for the option to use parallel processing and speed things up. To initialize parallel processing, use
+#'  `plan(multisession)` or `plan(multicore)` (depending on your operating system) prior to your piped code with the
+#'  `fn_once` or `fn_chain` functions. Note, parallel processing is best used when your code block takes more than a minute to run,
+#'  shorter run times will not benefit from parallel processing.
 #'
 #' @source Ozekin (1994), Sohn et al (2004), Song et al (1996), Galey et al (1997), Siddiqui et al (1994)
 #' @source See references list at: \url{https://github.com/BrownandCaldwell-Public/tidywater/wiki/References}
 #'
-#' @param water Source water object of class "water" created by \code{\link{define_water}}
+#' @param water Source water object of class "water" created by [define_water]
 #' @param dose Applied ozone dose (mg/L as O3). Results typically valid for 1-10 mg/L, but varies depending on model.
 #' @param time Reaction time (minutes). Results typically valid for 1-120 minutes, but varies depending on model.
 #' @param model Model to apply. One of c("Ozekin", "Sohn", "Song", "Galey", "Siddiqui")
 #' @examples
-#' example_dbp <- suppressWarnings(define_water(8, 20, 66, toc = 4, uv254 = .2, br = 50)) %>%
+#' example_dbp <- define_water(8, 20, 66, toc = 4, uv254 = .2, br = 50) %>%
 #'   ozonate_bromate(dose = 1.5, time = 5, model = "Ozekin")
-#' example_dbp <- suppressWarnings(define_water(7.5, 20, 66, toc = 4, uv254 = .2, br = 50)) %>%
+#' example_dbp <- define_water(7.5, 20, 66, toc = 4, uv254 = .2, br = 50) %>%
 #'   ozonate_bromate(dose = 3, time = 15, model = "Sohn")
 #'
 #' @export
-#' @returns A water class object with calculated bromate (ug/L).
+#' @returns `ozonate_bromate` returns a single water class object with calculated bromate (ug/L).
 #'
 ozonate_bromate <- function(water, dose, time, model = "Ozekin") {
   ammonia <- NULL # Quiet RCMD check global variable note
@@ -78,153 +89,29 @@ ozonate_bromate <- function(water, dose, time, model = "Ozekin") {
   return(water)
 }
 
-
-#' Apply `ozonate_bromate`function within a data frame and output a data frame
-#'
-#' This function allows \code{\link{ozonate_bromate}} to be added to a piped data frame.
-#' Its output is a data frame containing a bro3 column.
-#'
-#' The data input comes from a `water` class column, as initialized in \code{\link{define_water_chain}}.
-#'
-#' If the input data frame has a dose column (dose) or time column (time), the function will use those columns. Note:
-#' The function can only take dose and time inputs as EITHER a column or as function arguments, not both.
-#'
-#' tidywater functions cannot be added after this function because they require a `water` class input.
-#'
-#' For large datasets, using `fn_once` or `fn_chain` may take many minutes to run. These types of functions use the furrr package
-#' for the option to use parallel processing and speed things up. To initialize parallel processing, use
-#' `plan(multisession)` or `plan(multicore)` (depending on your operating system) prior to your piped code with the
-#' `fn_once` or `fn_chain` functions. Note, parallel processing is best used when your code block takes more than a minute to run,
-#' shorter run times will not benefit from parallel processing.
-#'
+#' @rdname ozonate_bromate
 #' @param df a data frame containing a water class column, which has already been computed using
-#' \code{\link{define_water_once}}. The df may include a column named for the applied chlorine dose (cl2),
-#' and a column for time in minutes.
-#' @param input_water name of the column of water class data to be used as the input for this function. Default is "defined_water".
-#' @param dose Applied ozone dose (mg/L as O3). Results typically valid for 1-10 mg/L, but varies depending on model.
-#' @param time Reaction time (minutes). Results typically valid for 1-120 minutes, but varies depending on model.
-#' @param model Model to apply, defaults to "Ozekin". One of c("Ozekin", "Sohn", "Song", "Galey", "Siddiqui")
-#'
-#' @seealso \code{\link{ozonate_bromate}}
-#'
-#' @examples
-#'
-#' library(purrr)
-#' library(furrr)
-#' library(tidyr)
-#' library(dplyr)
-#'
-#' example_df <- water_df %>%
-#'   mutate(br = 50) %>%
-#'   define_water_chain("raw") %>%
-#'   ozonate_bromate_once(input_water = "raw", dose = 3, time = 8)
-#'
-#' example_df <- water_df %>%
-#'   mutate(br = 50) %>%
-#'   define_water_chain("raw") %>%
-#'   mutate(
-#'     dose = c(seq(.5, 3, .5), seq(.5, 3, .5)),
-#'     time = 10
-#'   ) %>%
-#'   ozonate_bromate_once(input_water = "raw")
-#'
-#' example_df <- water_df %>%
-#'   mutate(br = 80) %>%
-#'   define_water_chain("raw") %>%
-#'   mutate(time = 8) %>%
-#'   ozonate_bromate_once(
-#'     input_water = "raw", dose = 6, model = "Sohn"
-#'   )
-#' \donttest{
-#' # Initialize parallel processing
-#' plan(multisession, workers = 2) # Remove the workers argument to use all available compute
-#' example_df <- water_df %>%
-#'   mutate(br = 50) %>%
-#'   define_water_chain() %>%
-#'   ozonate_bromate_once(input_water = "defined_water", dose = 4, time = 8)
-#'
-#' # Optional: explicitly close multisession processing
-#' plan(sequential)
-#' }
-#'
-#' @import dplyr
-#' @importFrom tidyr unnest
-#' @export
-#'
-#' @returns A data frame with updated bromate.
-
-ozonate_bromate_once <- function(df, input_water = "defined_water", dose = 0, time = 0,
-                                 model = "Ozekin") {
-  temp_o3 <- ozone <- NULL # Quiet RCMD check global variable note
-  output <- df %>%
-    ozonate_bromate_chain(
-      input_water = input_water, output_water = "temp_o3",
-      dose, time, model
-    ) %>%
-    mutate(ozone = furrr::future_map(temp_o3, convert_water)) %>%
-    unnest(ozone) %>%
-    select(-temp_o3)
-}
-
-#' Apply `ozonate_bromate` within a data frame and output a column of `water` class to be chained to other tidywater functions
-#'
-#' This function allows \code{\link{ozonate_bromate}} to be added to a piped data frame.
-#' Its output is a `water` class, and can therefore be used with "downstream" tidywater functions.
-#' The bro3 slot will be updated.
-#'
-#' The data input comes from a `water` class column, as initialized in \code{\link{define_water_chain}}.
-#'
-#' If the input data frame has a dose column (dose) or time column (time), the function will use those columns. Note:
-#' The function can only take dose and time inputs as EITHER a column or as function arguments, not both.
-#'
-#'  For large datasets, using `fn_once` or `fn_chain` may take many minutes to run. These types of functions use the furrr package
-#'  for the option to use parallel processing and speed things up. To initialize parallel processing, use
-#'  `plan(multisession)` or `plan(multicore)` (depending on your operating system) prior to your piped code with the
-#'  `fn_once` or `fn_chain` functions. Note, parallel processing is best used when your code block takes more than a minute to run,
-#'  shorter run times will not benefit from parallel processing.
-#'
-#' @param df a data frame containing a water class column, which has already been computed using
-#' \code{\link{define_water_chain}}. The df may include a column named for the applied ozone dose (dose),
+#' [define_water_once]. The df may include a column named for the applied chlorine dose (cl2),
 #' and a column for time in minutes.
 #' @param input_water name of the column of water class data to be used as the input for this function. Default is "defined_water".
 #' @param output_water name of the output column storing updated parameters with the class, water. Default is "ozonated_water".
-#' @param dose Applied ozone dose (mg/L as O3). Results typically valid for 1-10 mg/L, but varies depending on model.
-#' @param time Reaction time (minutes). Results typically valid for 1-120 minutes, but varies depending on model.
-#' @param model Model to apply, defaults to "Ozekin". One of c("Ozekin", "Sohn", "Song", "Galey", "Siddiqui")
-#'
-#' @seealso \code{\link{ozonate_bromate}}
-#'
 #' @examples
 #'
-#' library(purrr)
-#' library(furrr)
-#' library(tidyr)
 #' library(dplyr)
 #'
 #' example_df <- water_df %>%
-#'   mutate(br = 50) %>%
-#'   define_water_chain() %>%
-#'   ozonate_bromate_chain(dose = 4, time = 8)
-#'
-#' example_df <- water_df %>%
+#'   slice_head(n = 6) %>%
 #'   mutate(br = 50) %>%
 #'   define_water_chain() %>%
 #'   mutate(
-#'     dose = c(seq(.5, 3, .5), seq(.5, 3, .5)),
-#'     time = 30
+#'     dose = c(seq(.5, 3, .5)),
+#'     OzoneTime = 30
 #'   ) %>%
-#'   ozonate_bromate_chain()
-#'
-#' example_df <- water_df %>%
-#'   mutate(br = 80) %>%
-#'   define_water_chain() %>%
-#'   mutate(time = 8) %>%
-#'   ozonate_bromate_chain(
-#'     dose = 6, model = "Sohn"
-#'   )
+#'   ozonate_bromate_chain(time = OzoneTime, model = "Sohn")
 #'
 #' \donttest{
 #' # Initialize parallel processing
+#' library(furrr)
 #' plan(multisession, workers = 2) # Remove the workers argument to use all available compute
 #' example_df <- water_df %>%
 #'   mutate(br = 50) %>%
@@ -238,26 +125,31 @@ ozonate_bromate_once <- function(df, input_water = "defined_water", dose = 0, ti
 #' @import dplyr
 #' @export
 #'
-#' @returns A data frame containing a water class column with updated bro3.
+#' @returns `ozonate_bromate_chain` returns a data frame containing a water class column with updated bro3.
 
 ozonate_bromate_chain <- function(df, input_water = "defined_water", output_water = "ozonated_water",
-                                  dose = 0, time = 0, model = "Ozekin") {
-  ID <- NULL # Quiet RCMD check global variable note
-  arguments <- construct_helper(df, list("dose" = dose, "time" = time), list("model" = model))
+                                  dose = "use_col", time = "use_col", model = "use_col") {
+  validate_water_helpers(df, input_water)
+  # This allows for the function to process unquoted column names without erroring
+  dose <- tryCatch(dose, error = function(e) enquo(dose))
+  time <- tryCatch(time, error = function(e) enquo(time))
+  model <- tryCatch(model, error = function(e) enquo(model))
 
+  # This returns a dataframe of the input arguments and the correct column names for the others
+  arguments <- construct_helper(df, all_args = list("dose" = dose, "time" = time, "model" = model))
+
+  # Only join inputs if they aren't in existing dataframe
+  if (length(arguments$new_cols) > 0) {
+    df <- df %>%
+      cross_join(as.data.frame(arguments$new_cols))
+  }
   output <- df %>%
-    subset(select = !names(df) %in% c("dose", "time", "model")) %>%
-    mutate(
-      ID = row_number()
-    ) %>%
-    left_join(arguments, by = "ID") %>%
-    select(-ID) %>%
     mutate(!!output_water := furrr::future_pmap(
       list(
         water = !!as.name(input_water),
-        dose = dose,
-        time = time,
-        model = model
+        dose = !!as.name(arguments$final_names$dose),
+        time = !!as.name(arguments$final_names$time),
+        model = if (arguments$final_names$model %in% names(.)) !!sym(arguments$final_names$model) else rep("Ozekin", nrow(.))
       ),
       ozonate_bromate
     ))
