@@ -5,11 +5,12 @@
 solve_ph <- function(water, so4_dose = 0, na_dose = 0, ca_dose = 0, mg_dose = 0, cl_dose = 0) {
   # Correct eq constants
   ks <- correct_k(water)
+  gamma1 <- calculate_activity(1, water@is, water@temp)
 
   #### SOLVE FOR pH
   solve_h <- function(h, kw, so4_dose, tot_po4, h2po4_i, hpo4_i, po4_i, tot_co3, tot_ocl, tot_nh3, ocl_i, nh4_i,
                       alk_eq, na_dose, ca_dose, mg_dose, cl_dose) {
-    kw / h +
+    kw / (h * gamma1^2) +
       2 * so4_dose +
       tot_po4 * (calculate_alpha1_phosphate(h, ks) +
         2 * calculate_alpha2_phosphate(h, ks) +
@@ -44,6 +45,6 @@ solve_ph <- function(water, so4_dose = 0, na_dose = 0, ca_dose = 0, mg_dose = 0,
     cl_dose = cl_dose,
     tol = 1e-14
   )
-  phfinal <- -log10(root_h$root)
+  phfinal <- -log10(root_h$root * gamma1)
   return(round(phfinal, 2))
 }
