@@ -66,101 +66,65 @@ test_that("ozonate_bromate works.", {
 ################################################################################*
 ################################################################################*
 # ozonate_bromate helpers ----
-test_that("ozonate_bromate_once outputs are the same as base function, ozonate_bromate", {
-  water1 <- suppressWarnings(define_water(7.9, 20, 50,
-    tot_hard = 50, ca = 13,
-    na = 20, k = 20, cl = 30, so4 = 20,
-    tds = 200, cond = 100,
-    toc = 2, doc = 1.8, uv254 = 0.05, br = 50
-  )) %>%
-    ozonate_bromate(dose = 5, time = 8)
-
-  water2 <- suppressWarnings(water_df %>%
-    slice(1) %>%
-    mutate(br = 50) %>%
-    define_water_chain() %>%
-    ozonate_bromate_once(dose = 5, time = 8))
-
-  expect_equal(water1@bro3, water2$bro3)
-})
-
-# Check that output is a data frame
-
-test_that("ozonate_bromate_once is a data frame", {
-  water1 <- suppressWarnings(water_df %>%
-    slice(1) %>%
-    mutate(br = 50) %>%
-    define_water_chain() %>%
-    ozonate_bromate_once(
-      dose = 3, time = 10
-    ))
-
-  expect_true(is.data.frame(water1))
-})
-
-# Check ozonate_bromate_once can use a column or function argument for chemical dose
-
-test_that("ozonate_bromate_once can use a column or function argument for chemical dose", {
-  water1 <- suppressWarnings(water_df %>%
-    slice(1) %>%
-    mutate(br = 50) %>%
-    define_water_chain() %>%
-    ozonate_bromate_once(
-      dose = 3, time = 5
-    ))
-  water2 <- suppressWarnings(water_df %>%
-    slice(1) %>%
-    mutate(br = 50) %>%
-    define_water_chain() %>%
-    mutate(
-      dose = 3,
-      time = 5
-    ) %>%
-    ozonate_bromate_once())
-
-  water3 <- suppressWarnings(water_df %>%
-    slice(1) %>%
-    mutate(br = 50) %>%
-    define_water_chain() %>%
-    mutate(dose = 3) %>%
-    ozonate_bromate_once(time = 5))
-
-  expect_equal(water1$bro3, water2$bro3) # test different ways to input args
-  # Test that inputting dose and time separately (in column and as an argument) gives same results
-  expect_equal(water1$bro3, water3$bro3)
-})
 
 test_that("ozonate_bromate_chain outputs are the same as base function, ozonate_bromate", {
-  water1 <- suppressWarnings(define_water(7.9, 20, 50,
-    tot_hard = 50, ca = 13,
+  testthat::skip_on_cran()
+  water0 <- define_water(7.9, 20, 50,
+    tot_hard = 50, ca = 13, mg = 4,
     na = 20, k = 20, cl = 30, so4 = 20,
     tds = 200, cond = 100,
     toc = 2, doc = 1.8, uv254 = 0.05, br = 50
-  )) %>%
+  )
+  water1 <- water0 %>%
     ozonate_bromate(dose = 3, time = 5)
 
-  water2 <- suppressWarnings(water_df %>%
+  water2 <- water_df %>%
     mutate(br = 50) %>%
     slice(1) %>%
     define_water_chain() %>%
     ozonate_bromate_chain(dose = 3, time = 5, output_water = "ozone") %>%
-    pluck_water("ozone", c(
-      "bro3"
-    )))
+    pluck_water("ozone", "bro3")
+
+  models <- tibble(bromate_model = c("Ozekin", "Sohn", "Galey", "Siddiqui"))
+  doses <- tibble(dose = seq(1, 3, .5))
+  water3 <- water_df %>%
+    mutate(br = 50) %>%
+    slice(1) %>%
+    define_water_chain() %>%
+    cross_join(models) %>%
+    cross_join(doses) %>%
+    ozonate_bromate_chain(time = 5, model = bromate_model, output_water = "ozone") %>%
+    pluck_water("ozone", "bro3")
+
+  water4 <- ozonate_bromate(water0, dose = 2.5, time = 5, model = "Galey")
+
+  badmodels <- tibble(bromate_model = c("Ozekin", "Song", "Galey"))
+  water5 <- water_df %>%
+    mutate(br = 50) %>%
+    slice(1) %>%
+    define_water_chain() %>%
+    cross_join(badmodels) %>%
+    cross_join(doses)
+
+  expect_error(
+    ozonate_bromate_chain(water5, time = 5, model = bromate_model, output_water = "ozone")
+  )
 
   expect_equal(water1@bro3, water2$ozone_bro3)
+  expect_equal(water4@bro3, water3$ozone_bro3[14])
 })
 
 # Test that output is a column of water class lists, and changing the output column name works
 
 test_that("ozonate_bromate_chain output is list of water class objects, and can handle an ouput_water arg", {
+  testthat::skip_on_cran()
   water1 <- suppressWarnings(water_df %>%
     slice(1) %>%
     mutate(br = 60) %>%
     define_water_chain() %>%
     ozonate_bromate_chain(time = 5, dose = 3))
 
-  water2 <- purrr::pluck(water1, 5, 1)
+  water2 <- purrr::pluck(water1, "ozonated_water", 1)
 
   water3 <- suppressWarnings(water_df %>%
     mutate(br = 60) %>%
@@ -172,12 +136,13 @@ test_that("ozonate_bromate_chain output is list of water class objects, and can
     ozonate_bromate_chain(output_water = "diff_name"))
 
   expect_s4_class(water2, "water") # check class
-  expect_equal(names(water3[5]), "diff_name") # check if output_water arg works
+  expect_true(exists("diff_name", water3)) # check if output_water arg works
 })
 
 # Check ozonate_bromate_chain can use a column or function argument for chemical dose
 
 test_that("ozonate_bromate_chain can use a column or function argument for chemical dose, time", {
+  testthat::skip_on_cran()
   water1 <- suppressWarnings(water_df %>%
     slice(1) %>%
     mutate(br = 80) %>%
@@ -210,6 +175,7 @@ test_that("ozonate_bromate_chain can use a column or function argument for chemi
 })
 
 test_that("ozonate_bromate_chain multiple models", {
+  testthat::skip_on_cran()
   water1 <- suppressWarnings(water_df %>%
     slice(1) %>%
     mutate(br = 80) %>%
@@ -237,6 +203,7 @@ test_that("ozonate_bromate_chain multiple models", {
 })
 
 test_that("ozonate_bromate_chain errors with argument + column for same param", {
+  testthat::skip_on_cran()
   water <- water_df %>%
     define_water_chain("water")
   expect_error(water %>%
@@ -248,6 +215,7 @@ test_that("ozonate_bromate_chain errors with argument + column for same param",
 })
 
 test_that("ozonate_bromate_chain correctly handles arguments with multiple values", {
+  testthat::skip_on_cran()
   water <- water_df %>%
     mutate(br = 10) %>%
     slice(1:2) %>%
